@@ -53,6 +53,16 @@ struct NautilusFileDetails
 	NautilusDirectory *directory;
 	char *relative_uri;
 
+	/* Cached version of the display name, guaranteed UTF8 safe.
+	 * This is used a lot for sorting views.
+	 */
+	char *cached_display_name;
+	/* We cache the result of g_utf8_collate_key() on
+	 * cached_display_name in order to do quick sorting on
+	 * the display name
+	 */
+	char *display_name_collation_key;
+
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult get_info_error;
 
@@ -70,7 +80,7 @@ struct NautilusFileDetails
 
 	/* Info you might get from a link (.desktop, .directory or nautilus link) */
 	char *display_name;
-	char *custom_icon_uri;
+	char *custom_icon;
 	char *activation_uri;
 
 	/* The following is for file operations in progress. Since
@@ -132,6 +142,8 @@ gboolean      nautilus_file_get_date                       (NautilusFile
 							    NautilusDateType        date_type,
 							    time_t                 *date);
 void          nautilus_file_updated_deep_count_in_progress (NautilusFile           *file);
+void          nautilus_file_clear_cached_display_name      (NautilusFile           *file);
+
 
 /* Compare file's state with a fresh file info struct, return FALSE if
  * no change, update file and return TRUE if the file info contains
@@ -152,9 +164,10 @@ gboolean      nautilus_file_should_get_top_left_text       (NautilusFile
 void          nautilus_file_invalidate_attributes_internal (NautilusFile           *file,
 							    GList                  *file_attributes);
 GList *       nautilus_file_get_all_attributes             (void);
-
 gboolean      nautilus_file_is_self_owned                  (NautilusFile           *file);
+void          nautilus_file_invalidate_count_and_mime_list (NautilusFile           *file);
+gboolean      nautilus_file_rename_in_progress             (NautilusFile           *file);
 
-void          nautilus_file_invalidate_count_and_mime_list (NautilusFile              *file);
+GnomeVFSFileInfo *nautilus_file_peek_vfs_file_info         (NautilusFile           *file);
 
 #endif
