@@ -33,6 +33,7 @@
 #include "nautilus-file-dnd.h"
 #include "nautilus-icon-private.h"
 #include "nautilus-link.h"
+#include "nautilus-metadata.h"
 #include <eel/eel-background.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-glib-extensions.h>
@@ -73,10 +74,13 @@ static GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_ICON_DND_COLOR_TYPE, 0, NAUTILUS_ICON_DND_COLOR },
 	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
 	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD },
-	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND }
+	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND },
+	/* Must be last: */
+	{ NAUTILUS_ICON_DND_ROOTWINDOW_DROP_TYPE,  0, NAUTILUS_ICON_DND_ROOTWINDOW_DROP }
 };
 
 static GtkTargetList *drop_types_list = NULL;
+static GtkTargetList *drop_types_list_root = NULL;
 
 static EelCanvasItem *
 create_selection_shadow (NautilusIconContainer *container,
@@ -366,21 +370,43 @@ static void
 get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context, guint32 time)
 {
 	GList *target;
-
+	GtkTargetList *list;
+	
 	if (drop_types_list == NULL)
 		drop_types_list = gtk_target_list_new (drop_types,
-						       G_N_ELEMENTS (drop_types));
+						       G_N_ELEMENTS (drop_types) - 1);
+	if (drop_types_list_root == NULL)
+		drop_types_list_root = gtk_target_list_new (drop_types,
+							    G_N_ELEMENTS (drop_types));
 
+	if (nautilus_icon_container_get_is_desktop (NAUTILUS_ICON_CONTAINER (widget))) {
+		list = drop_types_list_root;
+	} else {
+		list = drop_types_list;
+	}
+	
 	for (target = context->targets; target != NULL; target = target->next) {
-		guint dummy_info;
+		guint info;
 		GdkAtom target_atom = GDK_POINTER_TO_ATOM (target->data);
+		NautilusDragInfo *drag_info;
 
-		if (gtk_target_list_find (drop_types_list, 
+		drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
+
+		if (gtk_target_list_find (list, 
 					  target_atom,
-					  &dummy_info)) {
-			gtk_drag_get_data (GTK_WIDGET (widget), context,
-					   target_atom,
-					   time);
+					  &info)) {
+			/* Don't get_data for rootwindow drops unless it's the actual drop */
+			if (info == NAUTILUS_ICON_DND_ROOTWINDOW_DROP &&
+			    !drag_info->drop_occured) {
+				/* We can't call get_data here, because that would
+				   make the source execute the rootwin action */
+				drag_info->got_drop_data_type = TRUE;
+				drag_info->data_type = NAUTILUS_ICON_DND_ROOTWINDOW_DROP;
+			} else {
+				gtk_drag_get_data (GTK_WIDGET (widget), context,
+						   target_atom,
+						   time);
+			}
 			break;
 		}
 	}
@@ -495,13 +521,56 @@ nautilus_icon_container_selection_items_local (NautilusIconContainer *container,
 	return result;
 }
 
+static GdkDragAction 
+get_background_drag_action (NautilusIconContainer *container, 
+			    GdkDragAction action)
+{
+	/* FIXME: This function is very FMDirectoryView specific, and
+	 * should be moved out of nautilus-icon-dnd.c */
+	GdkDragAction valid_actions;
+
+	if (action == GDK_ACTION_ASK) {
+		valid_actions = NAUTILUS_DND_ACTION_SET_AS_BACKGROUND;
+		if (g_object_get_data (G_OBJECT (eel_get_widget_background (GTK_WIDGET (container))), "is_desktop") == 0) {
+			valid_actions |= NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
+		}
+
+		action = nautilus_drag_drop_background_ask (valid_actions);
+	}
+
+	return action;
+}
+
+static void
+receive_dropped_color (NautilusIconContainer *container,
+		       int x, int y,
+		       GdkDragAction action,
+		       GtkSelectionData *data)
+{
+	action = get_background_drag_action (container, action);
+	
+	if (action > 0) {
+		eel_background_receive_dropped_color
+			(eel_get_widget_background (GTK_WIDGET (container)),
+			 GTK_WIDGET (container), 
+			 action, x, y, data);
+	}
+}
+
 /* handle dropped tile images */
 static void
-receive_dropped_tile_image (NautilusIconContainer *container, GtkSelectionData *data)
+receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction action, GtkSelectionData *data)
 {
 	g_assert (data != NULL);
-	eel_background_receive_dropped_background_image
-		(eel_get_widget_background (GTK_WIDGET (container)), data->data);
+
+	action = get_background_drag_action (container, action);
+
+	if (action > 0) {
+		eel_background_receive_dropped_background_image
+			(eel_get_widget_background (GTK_WIDGET (container)), 
+			 action, 
+			 data->data);
+	}
 }
 
 /* handle dropped keywords */
@@ -690,6 +759,9 @@ handle_local_move (NautilusIconContainer *container,
 	GList *moved_icons, *p;
 	NautilusDragSelectionItem *item;
 	NautilusIcon *icon;
+	NautilusFile *file;
+	char *screen_string;
+	GdkScreen *screen;
 
 	if (container->details->auto_layout) {
 		if (!confirm_switch_to_manual_layout (container)) {
@@ -698,6 +770,7 @@ handle_local_move (NautilusIconContainer *container,
 		nautilus_icon_container_freeze_icon_positions (container);
 	}
 
+
 	/* Move and select the icons. */
 	moved_icons = NULL;
 	for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
@@ -706,6 +779,29 @@ handle_local_move (NautilusIconContainer *container,
 		icon = nautilus_icon_container_get_icon_by_uri
 			(container, item->uri);
 
+		if (icon == NULL) {
+			/* probably dragged from another screen.  Add it to
+			 * this screen
+			 */
+
+			file = nautilus_file_get (item->uri);
+
+			screen = gtk_widget_get_screen (GTK_WIDGET (container));
+			screen_string = g_strdup_printf ("%d",
+						gdk_screen_get_number (screen));
+			nautilus_file_set_metadata (file,
+					NAUTILUS_METADATA_KEY_SCREEN,
+					NULL, screen_string);
+
+			g_free (screen_string);
+
+			nautilus_icon_container_add (container,
+					NAUTILUS_ICON_CONTAINER_ICON_DATA (file));
+			
+			icon = nautilus_icon_container_get_icon_by_uri
+				(container, item->uri);
+		}
+
 		if (item->got_icon_position) {
 			nautilus_icon_container_move_icon
 				(container, icon,
@@ -736,7 +832,7 @@ handle_nonlocal_move (NautilusIconContainer *container,
 	if (container->details->dnd_info->drag_info.selection_list == NULL) {
 		return;
 	}
-	
+
 	source_uris = NULL;
 	for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
 		/* do a shallow copy of all the uri strings of the copied files */
@@ -890,6 +986,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 		selected_item = container->details->dnd_info->drag_info.selection_list->data;
 		eel_background_receive_dropped_background_image
 			(eel_get_widget_background (GTK_WIDGET (container)),
+			 context->action,
 			 selected_item->uri);
 		return;
 	}
@@ -934,7 +1031,7 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 	gboolean icon_hit;
 	NautilusIcon *icon;
 	double world_x, world_y;
-	
+
 	icon_hit = FALSE;
 	if (!container->details->dnd_info->drag_info.got_drop_data_type) {
 		/* drag_data_received_callback didn't get called yet */
@@ -983,6 +1080,7 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 	
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_URL:
+	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = context->suggested_action;
 		break;
 
@@ -1297,7 +1395,7 @@ drag_data_received_callback (GtkWidget *widget,
     	NautilusDragInfo *drag_info;
 	EelBackground *background;
 	gboolean success;
-	
+
 	drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
 
 	drag_info->got_drop_data_type = TRUE;
@@ -1326,9 +1424,11 @@ drag_data_received_callback (GtkWidget *widget,
 			drag_info->selection_data = gtk_selection_data_copy (data);
 		}
 		break;
+	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
+		/* Do nothing, this won't even happen, since we don't want to call get_data twice */
+		break;
 	}
 
-
 	/* this is the second use case of this callback.
 	 * we have to do the actual work for the drop.
 	 */
@@ -1342,14 +1442,16 @@ drag_data_received_callback (GtkWidget *widget,
 				 context, x, y);
 			break;
 		case NAUTILUS_ICON_DND_COLOR:
-			eel_background_receive_dropped_color
-				(eel_get_widget_background (widget),
-				 widget, x, y, data);
+			receive_dropped_color (NAUTILUS_ICON_CONTAINER (widget),
+					       x, y,
+					       context->action,
+					       data);
 			success = TRUE;
 			break;
 		case NAUTILUS_ICON_DND_BGIMAGE:
 			receive_dropped_tile_image
 				(NAUTILUS_ICON_CONTAINER (widget),
+				 context->action,
 				 data);
 			break;
 		case NAUTILUS_ICON_DND_KEYWORD:
@@ -1371,6 +1473,9 @@ drag_data_received_callback (GtkWidget *widget,
 			}
 			gtk_drag_finish (context, FALSE, FALSE, time);			
 			break;
+		case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
+			/* Do nothing, everything is done by the sender */
+			break;
 		}
 		gtk_drag_finish (context, success, FALSE, time);
 		
@@ -1403,6 +1508,8 @@ void
 nautilus_icon_dnd_init (NautilusIconContainer *container,
 			GdkBitmap *stipple)
 {
+	int n_elements;
+	
 	g_return_if_fail (container != NULL);
 	g_return_if_fail (NAUTILUS_IS_ICON_CONTAINER (container));
 
@@ -1415,9 +1522,14 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 	 * (But not a source, as drags starting from this widget will be
          * implemented by dealing with events manually.)
 	 */
+	n_elements = G_N_ELEMENTS (drop_types);
+	if (!nautilus_icon_container_get_is_desktop (container)) {
+		/* Don't set up rootwindow drop */
+		n_elements -= 1;
+	}
 	gtk_drag_dest_set (GTK_WIDGET (container),
 			   0,
-			   drop_types, G_N_ELEMENTS (drop_types),
+			   drop_types, n_elements,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
 	/* Messages for outgoing drag. */
