@@ -20,20 +20,15 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
   
-   Author: Darin Adler <darin@eazel.com>
+   Author: Darin Adler <darin@bentspoon.com>
 */
 
 #ifndef NAUTILUS_ICON_FACTORY_PRIVATE_H
 #define NAUTILUS_ICON_FACTORY_PRIVATE_H
 
-#include "nautilus-icon-factory.h"
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
-/* For now, images are used themselves as thumbnails when they are
- * below this threshold size. Later we might have to have a more
- * complex rule about when to use an image for itself.
- */
-#define SELF_THUMBNAIL_SIZE_THRESHOLD   16384
-
-void nautilus_icon_factory_remove_by_uri (const char *uri);
+/* Convenience routine to return the appropriate thumbnail frame. */
+GdkPixbuf *nautilus_icon_factory_get_thumbnail_frame (void);
 
 #endif /* NAUTILUS_ICON_FACTORY_PRIVATE_H */
