@@ -22,7 +22,7 @@
  *
  *  Authors: Elliot Lee <sopwith@redhat.com>
  *           Maciej Stachowiak <mjs@eazel.com>
- *           Darin Adler <darin@eazel.com>
+ *           Darin Adler <darin@bentspoon.com>
  *
  */
 
@@ -66,6 +66,8 @@ get_view_result_from_gnome_vfs_result (GnomeVFSResult gnome_vfs_result)
                 return NAUTILUS_DETERMINE_VIEW_HOST_NOT_FOUND;
 	case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
 		return NAUTILUS_DETERMINE_VIEW_HOST_HAS_NO_ADDRESS;
+        case GNOME_VFS_ERROR_NO_MASTER_BROWSER:
+                return NAUTILUS_DETERMINE_VIEW_NO_MASTER_BROWSER;
         case GNOME_VFS_ERROR_GENERIC:
         case GNOME_VFS_ERROR_INTERNAL:
                 /* These two have occurred at least once in the web browser component */
@@ -96,7 +98,7 @@ got_file_info_callback (NautilusFile *file,
         GnomeVFSResult vfs_result_code;
         NautilusDetermineViewResult result_code;
         NautilusViewIdentifier *default_id;
-        OAF_ServerInfo *default_component;
+        Bonobo_ServerInfo *default_component;
         
         handle = (NautilusDetermineViewHandle *) callback_data;
         
