@@ -33,7 +33,7 @@
 #include "nautilus-bookmark-list.h"
 #include "nautilus-bookmark-parsing.h"
 #include "nautilus-bookmarks-window.h"
-#include "nautilus-preferences-dialog.h"
+#include "nautilus-file-management-properties.h"
 #include "nautilus-property-browser.h"
 #include "nautilus-signaller.h"
 #include "nautilus-switchable-navigation-bar.h"
@@ -50,6 +50,8 @@
 #include <eel/eel-xml-extensions.h>
 #include <libxml/parser.h>
 #include <gtk/gtkmain.h>
+#include <libegg/egg-screen-help.h>
+#include <libgnome/gnome-help.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-about.h>
@@ -63,7 +65,6 @@
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus/nautilus-bonobo-ui.h>
 
-
 #ifdef ENABLE_PROFILER
 #include "nautilus-profiler.h"
 #endif
@@ -90,12 +91,20 @@
 #define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/menu/Bookmarks/Bookmarks Placeholder"
 
 #define COMMAND_PATH_CLOSE_WINDOW			"/commands/Close"
+#define COMMAND_SHOW_HIDE_SIDEBAR                       "/commands/Show Hide Sidebar"
+#define COMMAND_SHOW_HIDE_TOOLBAR                       "/commands/Show Hide Toolbar"
+#define COMMAND_SHOW_HIDE_LOCATION_BAR                  "/commands/Show Hide Location Bar"
+#define COMMAND_SHOW_HIDE_STATUS_BAR                    "/commands/Show Hide Statusbar"
+
+#define ID_SHOW_HIDE_SIDEBAR                            "Show Hide Sidebar"
+#define ID_SHOW_HIDE_TOOLBAR                            "Show Hide Toolbar"
+#define ID_SHOW_HIDE_LOCATION_BAR                       "Show Hide Location Bar"
+#define ID_SHOW_HIDE_STATUS_BAR                         "Show Hide Statusbar"
 
-#define USER_MANUAL_URI		"help:nautilus-user-manual"
-#define QUICK_REFERENCE_URI	"help:nautilus-quick-reference"
-#define CUSTOMER_FEEDBACK_URI	"http://www.eazel.com/feedback.html"
 #define START_HERE_URI          "start-here:"
 
+#define RESPONSE_FORGET		1000
+
 static GtkWindow *bookmarks_window = NULL;
 
 static void                  append_bookmark_to_menu                       (NautilusWindow   *window,
@@ -161,7 +170,6 @@ bookmark_holder_free (BookmarkHolder *bookmark_holder)
  * don't want other code relying on their existence.
  */
 
-#define NAUTILUS_MENU_PATH_CUSTOMIZE_ITEM			"/menu/Edit/Customization"
 
 static void
 bookmark_holder_free_cover (gpointer callback_data, GClosure *closure)
@@ -178,7 +186,9 @@ file_menu_new_window_callback (BonoboUIComponent *component,
 	NautilusWindow *new_window;
 
 	current_window = NAUTILUS_WINDOW (user_data);
-	new_window = nautilus_application_create_window (current_window->application);
+	new_window = nautilus_application_create_window (
+				current_window->application,
+				gtk_window_get_screen (GTK_WINDOW (current_window)));
 	nautilus_window_go_home (new_window);
 }
 
@@ -317,10 +327,19 @@ go_menu_start_here_callback (BonoboUIComponent *component,
 			       START_HERE_URI);
 }
 
+static void
+go_menu_go_to_trash_callback (BonoboUIComponent *component, 
+			      gpointer user_data, 
+			      const char *verb) 
+{
+	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
+			       EEL_TRASH_URI);
+}
+
 static void
 forget_history_if_yes (GtkDialog *dialog, int response, gpointer callback_data)
 {
-	if (response == GTK_RESPONSE_YES) {
+	if (response == RESPONSE_FORGET) {
 		nautilus_forget_history ();
 	}
 	gtk_object_destroy (GTK_OBJECT (dialog));
@@ -343,19 +362,24 @@ forget_history_if_confirmed (NautilusWindow *window)
 		prompt = g_strdup (_("Are you sure you want to forget history? "
 				     "If you do, you will be doomed to repeat it."));
 	} else {
-		prompt = g_strdup (_("Are you sure you want Nautilus to forget "
-				     "which locations you have visited?"));
+		prompt = g_strdup (_("Are you sure you want to clear the list "
+				     "of locations you have visited?"));
 	}
 					   
-	dialog = eel_show_yes_no_dialog (prompt,
-					 _("Forget History?"), _("Forget"), GTK_STOCK_CANCEL,
-					 GTK_WINDOW (window));
+	dialog = eel_create_question_dialog (prompt,
+					     _("Clear History"), 
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     GTK_STOCK_CLEAR, RESPONSE_FORGET,
+					     GTK_WINDOW (window));
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+
 	g_free (prompt);					 
 	
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (forget_history_if_yes), NULL);
-	
-	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_NO);
+
+	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
 }
 
 static void
@@ -375,62 +399,94 @@ view_menu_reload_callback (BonoboUIComponent *component,
 }
 
 static void
-view_menu_show_hide_sidebar_callback (BonoboUIComponent *component, 
-			              gpointer user_data, 
-			              const char *verb) 
+view_menu_show_hide_sidebar_state_changed_callback (BonoboUIComponent *component, 
+						    const char *path,
+						    Bonobo_UIComponent_EventType type,
+						    const char *state,
+						    gpointer user_data)
 {
 	NautilusWindow *window;
 
 	window = NAUTILUS_WINDOW (user_data);
-	if (nautilus_window_sidebar_showing (window)) {
-		nautilus_window_hide_sidebar (window);
-	} else {
+
+        if (strcmp (state, "") == 0) {
+                /* State goes blank when component is removed; ignore this. */
+                return;
+        }
+
+	if (!strcmp (state, "1")) {
 		nautilus_window_show_sidebar (window);
+	} else {
+		nautilus_window_hide_sidebar (window);
 	}
 }
 
 static void
-view_menu_show_hide_toolbar_callback (BonoboUIComponent *component, 
-			               gpointer user_data, 
-			               const char *verb) 
+view_menu_show_hide_toolbar_state_changed_callback (BonoboUIComponent *component, 
+						    const char *path,
+						    Bonobo_UIComponent_EventType type,
+						    const char *state,
+						    gpointer user_data)
 {
 	NautilusWindow *window;
 
 	window = NAUTILUS_WINDOW (user_data);
-	if (nautilus_window_toolbar_showing (window)) {
-		nautilus_window_hide_toolbar (window);
-	} else {
+
+        if (strcmp (state, "") == 0) {
+                /* State goes blank when component is removed; ignore this. */
+                return;
+        }
+
+	if (!strcmp (state, "1")) {
 		nautilus_window_show_toolbar (window);
+	} else {
+		nautilus_window_hide_toolbar (window);
 	}
 }
 
 static void
-view_menu_show_hide_location_bar_callback (BonoboUIComponent *component, 
-			                   gpointer user_data, 
-			                   const char *verb) 
+view_menu_show_hide_location_bar_state_changed_callback (BonoboUIComponent *component, 
+							 const char *path,
+							 Bonobo_UIComponent_EventType type,
+							 const char *state,
+							 gpointer user_data)
 {
 	NautilusWindow *window;
 
 	window = NAUTILUS_WINDOW (user_data);
-	if (nautilus_window_location_bar_showing (window)) {
-		nautilus_window_hide_location_bar (window);
-	} else {
+
+        if (strcmp (state, "") == 0) {
+                /* State goes blank when component is removed; ignore this. */
+                return;
+        }
+
+	if (!strcmp (state, "1")) {
 		nautilus_window_show_location_bar (window);
+	} else {
+		nautilus_window_hide_location_bar (window);
 	}
 }
 
 static void
-view_menu_show_hide_status_bar_callback (BonoboUIComponent *component, 
-			                 gpointer user_data, 
-			                 const char *verb) 
+view_menu_show_hide_statusbar_state_changed_callback (BonoboUIComponent *component, 
+						      const char *path,
+						      Bonobo_UIComponent_EventType type,
+						      const char *state,
+						      gpointer user_data)
 {
 	NautilusWindow *window;
 
 	window = NAUTILUS_WINDOW (user_data);
-	if (nautilus_window_status_bar_showing (window)) {
-		nautilus_window_hide_status_bar (window);
-	} else {
+
+        if (strcmp (state, "") == 0) {
+                /* State goes blank when component is removed; ignore this. */
+                return;
+        }
+
+	if (!strcmp (state, "1")) {
 		nautilus_window_show_status_bar (window);
+	} else {
+		nautilus_window_hide_status_bar (window);
 	}
 }
 
@@ -442,29 +498,20 @@ nautilus_window_update_show_hide_menu_items (NautilusWindow *window)
 	nautilus_window_ui_freeze (window);
 
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
-	nautilus_bonobo_set_label (window->details->shell_ui,
-				   MENU_PATH_SHOW_HIDE_STATUS_BAR,
-				   nautilus_window_status_bar_showing (window)
-				   ? _("Hide St_atus Bar")
-				   : _("Show St_atus Bar"));
 		
-	nautilus_bonobo_set_label (window->details->shell_ui,
-				   MENU_PATH_SHOW_HIDE_SIDEBAR,
-				   nautilus_window_sidebar_showing (window)
-				   ? _("Hide _Sidebar")
-				   : _("Show _Sidebar"));
+	nautilus_bonobo_set_toggle_state (window->details->shell_ui,
+					  COMMAND_SHOW_HIDE_SIDEBAR,
+					  nautilus_window_sidebar_showing (window));
+	nautilus_bonobo_set_toggle_state (window->details->shell_ui,
+					  COMMAND_SHOW_HIDE_TOOLBAR,
+					  nautilus_window_toolbar_showing (window));
+	nautilus_bonobo_set_toggle_state (window->details->shell_ui,
+					  COMMAND_SHOW_HIDE_LOCATION_BAR,
+					  nautilus_window_location_bar_showing (window));
+	nautilus_bonobo_set_toggle_state (window->details->shell_ui,
+					  COMMAND_SHOW_HIDE_STATUS_BAR,
+					  nautilus_window_status_bar_showing (window));
 
-	nautilus_bonobo_set_label (window->details->shell_ui,
-				   MENU_PATH_SHOW_HIDE_TOOLBAR,
-				   nautilus_window_toolbar_showing (window)
-				   ? _("Hide _Toolbar")
-				   : _("Show _Toolbar"));
-		
-	nautilus_bonobo_set_label (window->details->shell_ui,
-				   MENU_PATH_SHOW_HIDE_LOCATION_BAR,
-				   nautilus_window_location_bar_showing (window)
-				   ? _("Hide Location _Bar")
-				   : _("Show Location _Bar"));
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 
 	nautilus_window_ui_thaw (window);
@@ -519,19 +566,36 @@ bookmarks_menu_edit_bookmarks_callback (BonoboUIComponent *component,
 }
 
 static void
-user_level_customize_callback (BonoboUIComponent *component, 
-			       gpointer user_data, 
-			       const char *verb)
+preferences_respond_callback (GtkDialog *dialog,
+			      gint response_id)
 {
-	nautilus_preferences_dialog_show ();
+	if (response_id == GTK_RESPONSE_CLOSE) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+	}
 }
 
 static void
-customize_callback (BonoboUIComponent *component, 
-		    gpointer user_data, 
-		    const char *verb)
+preferences_callback (BonoboUIComponent *component, 
+		      gpointer user_data, 
+		      const char *verb)
 {
-	nautilus_property_browser_show ();
+	GtkWindow *window;
+
+	window = GTK_WINDOW (user_data);
+
+	nautilus_file_management_properties_dialog_show (G_CALLBACK (preferences_respond_callback));
+}
+
+static void
+backgrounds_and_emblems_callback (BonoboUIComponent *component, 
+				  gpointer user_data, 
+				  const char *verb)
+{
+	GtkWindow *window;
+
+	window = GTK_WINDOW (user_data);
+
+	nautilus_property_browser_show (gtk_window_get_screen (window));
 }
 
 static void
@@ -540,12 +604,14 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 			           const char *verb)
 {
 	static GtkWidget *about = NULL;
-	char *build_message;
 	const char *authors[] = {
+		"Alexander Larsson",
 		"Ali Abdin",
+		"Anders Carlsson",
 		"Andy Hertzfeld",
 		"Arlo Rose",
 		"Darin Adler",
+		"David Camp",
 		"Eli Goldberg",
 		"Elliot Lee",
 		"Eskil Heyn Olsen",
@@ -554,10 +620,13 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 		"George Lebl",
 		"Ian McKellar",
 		"J Shane Culpepper",
+		"James Willcox",
+		"Jan Arne Petersen",
 		"John Harper",
 		"John Sullivan",
 		"Josh Barrow",
 		"Maciej Stachowiak",
+		"Mark McLoughlin",
 		"Mathieu Lacage",
 		"Mike Engber",
 		"Mike Fleming",
@@ -576,18 +645,6 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 	const char *locale;
 
 	if (about == NULL) {
-		/* The time stamp overrides the build message, because
-		 * the time stamp should only be set on Tinderbox for
-		 * hourly builds.
-		 */
-		build_message = nautilus_get_build_time_stamp ();
-		if (build_message == NULL) {
-			build_message = nautilus_get_build_message ();
-			if (build_message == NULL) {
-				build_message = g_strdup ("");
-			}
-		}
-		
 		/* We could probably just put a translation in en_US
 		 * instead of doing this mess, but I got this working
 		 * and I don't feel like fiddling with it any more.
@@ -611,7 +668,8 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 		 * which will be displayed at the bottom of the about
 		 * box to give credit to the translator(s).
 		 */
-		translator_credits = _("Translator Credits");
+		translator_credits = (strcmp (_("Translator Credits"), "Translator Credits") == 0) ?
+			NULL : _("Translator Credits");
 		
 		about = gnome_about_new (_("Nautilus"),
 					 VERSION,
@@ -626,8 +684,6 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 					 NULL);
 		gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (user_data));
 		
-		g_free (build_message);
-
 		eel_add_weak_pointer (&about);
 	}
 	
@@ -639,24 +695,32 @@ help_menu_nautilus_manual_callback (BonoboUIComponent *component,
 			              gpointer user_data, 
 			              const char *verb)
 {
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data), USER_MANUAL_URI);
-}
+	NautilusWindow *window;
+	GError *error;
+	GtkWidget *dialog;
 
-static void
-help_menu_nautilus_quick_reference_callback (BonoboUIComponent *component, 
-			              	     gpointer user_data, 
-			              	     const char *verb)
-{
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data), QUICK_REFERENCE_URI);
-}
+	error = NULL;
+	window = NAUTILUS_WINDOW (user_data);
 
-static void
-help_menu_nautilus_feedback_callback (BonoboUIComponent *component, 
-			              gpointer user_data, 
-			              const char *verb)
-{
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       CUSTOMER_FEEDBACK_URI);
+	egg_help_display_desktop_on_screen (
+		NULL, "user-guide", "wgosnautilus.xml", "gosnautilus-21",
+		gtk_window_get_screen (GTK_WINDOW (window)), &error);
+
+	if (error) {
+		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("There was an error displaying help: \n%s"),
+						 error->message);
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_widget_show (dialog);
+		g_error_free (error);
+	}
 }
 
 static void
@@ -788,6 +852,7 @@ append_bookmark_to_menu (NautilusWindow *window,
 	nautilus_window_ui_thaw (window);
 }
 
+#ifdef WEB_NAVIGATION_ENABLED
 static char *
 get_static_bookmarks_file_path (void)
 {
@@ -805,7 +870,7 @@ get_static_bookmarks_file_path (void)
 	g_free (user_directory_path);
 	
 	/* get the mod date of the built-in static bookmarks file */
-	built_in_xml_file_path = nautilus_make_path (NAUTILUS_DATADIR, STATIC_BOOKMARKS_FILE_NAME);
+	built_in_xml_file_path = g_build_filename (NAUTILUS_DATADIR, STATIC_BOOKMARKS_FILE_NAME, NULL);
 	built_in_exists = g_file_test (built_in_xml_file_path, G_FILE_TEST_EXISTS);
 	
 	/* if we only have one file, return its path as the one to use */
@@ -851,6 +916,7 @@ get_static_bookmarks_file_path (void)
 	
 	return result;
 }
+#endif
 
 static void
 append_separator (NautilusWindow *window, const char *path)
@@ -883,7 +949,7 @@ create_menu_item_from_node (NautilusWindow *window,
 
 	if (strcmp (node->name, "bookmark") == 0) {
 		bookmark = nautilus_bookmark_new_from_node (node);
-		append_bookmark_to_menu (window, bookmark, menu_path, *index, TRUE);		
+		append_bookmark_to_menu (window, bookmark, menu_path, *index, TRUE);
 		g_object_unref (bookmark);
 	} else if (strcmp (node->name, "separator") == 0) {
 		append_separator (window, menu_path);
@@ -912,6 +978,7 @@ create_menu_item_from_node (NautilusWindow *window,
 	nautilus_window_ui_thaw (window);
 }
 
+#ifdef WEB_NAVIGATION_ENABLED
 static void
 append_static_bookmarks (NautilusWindow *window, const char *menu_path)
 {
@@ -939,6 +1006,7 @@ append_static_bookmarks (NautilusWindow *window, const char *menu_path)
 	
 	xmlFreeDoc(doc);
 }
+#endif
 
 static NautilusBookmarkList *bookmarks = NULL;
 
@@ -999,8 +1067,13 @@ add_bookmark_for_current_location (NautilusWindow *window)
 static void
 edit_bookmarks (NautilusWindow *window)
 {
-        gtk_window_present
-		(get_or_create_bookmarks_window (G_OBJECT (window)));
+	GtkWindow *dialog;
+
+	dialog = get_or_create_bookmarks_window (G_OBJECT (window));
+
+	gtk_window_set_screen (
+		dialog, gtk_window_get_screen (GTK_WINDOW (window)));
+        gtk_window_present (dialog);
 }
 
 void
@@ -1017,15 +1090,21 @@ refresh_bookmarks_menu (NautilusWindow *window)
 	/* Unregister any pending call to this function. */
 	nautilus_window_remove_bookmarks_menu_callback (window);
 
+	g_object_ref (G_OBJECT (window));
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
-	nautilus_window_remove_bookmarks_menu_items (window);				
 
+	nautilus_window_remove_bookmarks_menu_items (window);
+
+#ifdef WEB_NAVIGATION_ENABLED
 	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS)) {
 		append_static_bookmarks (window, MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER);
 	}
+#endif
 
 	append_dynamic_bookmarks (window);
+
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
+	g_object_unref (G_OBJECT (window));
 }
 
 /**
@@ -1052,11 +1131,13 @@ nautilus_window_initialize_bookmarks_menu (NautilusWindow *window)
 		
 	/* Recreate dynamic part of menu if bookmark list changes */
 	g_signal_connect_object (get_bookmark_list (), "contents_changed",
-				 G_CALLBACK (schedule_refresh_bookmarks_menu), window, G_CONNECT_SWAPPED);
+				 G_CALLBACK (schedule_refresh_bookmarks_menu),
+				 window, G_CONNECT_SWAPPED);
 
 	/* Recreate static & dynamic parts of menu if icon theme changes */
 	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
-				 G_CALLBACK (schedule_refresh_bookmarks_menu), window, G_CONNECT_SWAPPED);
+				 G_CALLBACK (schedule_refresh_bookmarks_menu),
+				 window, G_CONNECT_SWAPPED);
 }
 
 /**
@@ -1094,19 +1175,16 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 		BONOBO_UI_VERB ("Toggle Find Mode", toolbar_toggle_find_mode_callback),
 #endif
 		BONOBO_UI_VERB ("Undo", edit_menu_undo_callback),
-		BONOBO_UI_VERB ("Customize", customize_callback),
+		BONOBO_UI_VERB ("Backgrounds and Emblems", backgrounds_and_emblems_callback),
 		BONOBO_UI_VERB ("Back", go_menu_back_callback),
 		BONOBO_UI_VERB ("Forward", go_menu_forward_callback),
 		BONOBO_UI_VERB ("Up", go_menu_up_callback),
 		BONOBO_UI_VERB ("Home", go_menu_home_callback),
 		BONOBO_UI_VERB ("Start Here", go_menu_start_here_callback),
+		BONOBO_UI_VERB ("Go to Trash", go_menu_go_to_trash_callback),
 		BONOBO_UI_VERB ("Go to Location", go_menu_location_callback),
-		BONOBO_UI_VERB ("Forget History", go_menu_forget_history_callback),
+		BONOBO_UI_VERB ("Clear History", go_menu_forget_history_callback),
 		BONOBO_UI_VERB ("Reload", view_menu_reload_callback),
-		BONOBO_UI_VERB ("Show Hide Sidebar", view_menu_show_hide_sidebar_callback),
-		BONOBO_UI_VERB ("Show Hide Toolbar", view_menu_show_hide_toolbar_callback),
-		BONOBO_UI_VERB ("Show Hide Location Bar", view_menu_show_hide_location_bar_callback),
-		BONOBO_UI_VERB ("Show Hide Statusbar", view_menu_show_hide_status_bar_callback),
 		BONOBO_UI_VERB ("Zoom In", view_menu_zoom_in_callback),
 		BONOBO_UI_VERB ("Zoom Out", view_menu_zoom_out_callback),
 		BONOBO_UI_VERB ("Zoom Normal", view_menu_zoom_normal_callback),
@@ -1123,9 +1201,7 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 
 		BONOBO_UI_VERB ("About Nautilus", help_menu_about_nautilus_callback),
 		BONOBO_UI_VERB ("Nautilus Manual", help_menu_nautilus_manual_callback),
-		BONOBO_UI_VERB ("Nautilus Quick Reference", help_menu_nautilus_quick_reference_callback),
-		BONOBO_UI_VERB ("Nautilus Feedback", help_menu_nautilus_feedback_callback),
-		BONOBO_UI_VERB ("User Level Customization", user_level_customize_callback),
+		BONOBO_UI_VERB ("Preferences", preferences_callback),
 		BONOBO_UI_VERB ("Stop", stop_button_callback),
 
 		BONOBO_UI_VERB_END
@@ -1135,9 +1211,30 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 
+	nautilus_window_update_show_hide_menu_items (window);
+
 	bonobo_ui_component_add_verb_list_with_data (window->details->shell_ui, verbs, window);
 
-        nautilus_window_update_show_hide_menu_items (window);
+	bonobo_ui_component_add_listener
+		(window->details->shell_ui,
+		 ID_SHOW_HIDE_SIDEBAR,
+		 view_menu_show_hide_sidebar_state_changed_callback, 
+		 window);
+	bonobo_ui_component_add_listener
+		(window->details->shell_ui,
+		 ID_SHOW_HIDE_TOOLBAR,
+		 view_menu_show_hide_toolbar_state_changed_callback, 
+		 window); 
+	bonobo_ui_component_add_listener
+		(window->details->shell_ui,
+		 ID_SHOW_HIDE_LOCATION_BAR,
+		 view_menu_show_hide_location_bar_state_changed_callback, 
+		 window);
+	bonobo_ui_component_add_listener
+		(window->details->shell_ui,
+		 ID_SHOW_HIDE_STATUS_BAR,
+		 view_menu_show_hide_statusbar_state_changed_callback, 
+		 window);
 
 	/* Register to catch Bonobo UI events so we can notice View As changes */
 	g_signal_connect_object (window->details->shell_ui, "ui_event", 
@@ -1294,7 +1391,7 @@ refresh_go_menu (NautilusWindow *window)
 
 	/* Add in a new set of history items. */
 	for (node = nautilus_get_history_list (), index = 0;
-	     node != NULL;
+	     node != NULL && index < 10;
 	     node = node->next, index++) {
 		append_bookmark_to_menu (window,
 					 NAUTILUS_BOOKMARK (node->data),
@@ -1328,5 +1425,3 @@ schedule_refresh_go_menu (NautilusWindow *window)
                                         window);
 	}	
 }
-
-
